--
-- _builddeb.lua
-- builddeb main lua script
-- Copyright (c) 2010 pasgui
--
-- Hash: $Format:%h$
-- Date: $Format:%ci$
--

local shorthelp     = "Type 'builddeb --help' for help"
local versionhelp   = "builddeb (Debian repository builder) "

--
-- Script-side program entry point.
--
function _builddeb_main(scriptpath)

  -- if running off the disk (in debug mode), load everything
  -- listed in _manifest.lua; the list divisions make sure
  -- everything gets initialized in the proper order.
  if (scriptpath) then
    local scripts  = dofile(scriptpath .. "/_manifest.lua")
    for _,v in pairs(scripts) do
      dofile(scriptpath .. "/" .. v)
    end
  end

  -- Process special options
  if (_OPTIONS["version"]) then
    print(versionhelp .. _BUILDDEB_VERSION)
    return 1
  end

-- print help page then exit
  if (_OPTIONS["help"]) then
    builddeb.showhelp()
    return 1
  end

  -- If no action was specified, show a short help message
  if (not _ACTION) then
    print(shorthelp)
    return 1
  end

  -- checks action
  if string.find("all build web", _ACTION, 1, true) == nil then
    builddeb.showhelp()
    return 1
  end

  -- load configuration
  local filename = _OPTIONS["conf"] or "ubuntu.xml"
  if os.exists(filename) == false then
    error("Error: configuration file '"..filename.."' not found!", 0)
  end
  configuration.load(filename)

  -- build debian repository
  if string.find("all build", _ACTION, 1, true) then
    -- build repository
    repository.build()
  end

  -- build web folder
  if string.find("all web", _ACTION, 1, true) then
    -- get path information from configuration
    local path = configuration.path
    -- check if the ubuntu directory exists
    if os.exists(path.ubuntu) == false then
      error("Error: ubuntu directory '"..path.ubuntu.."' not found!", 0)
    end
    -- build web folder
    web.build()
    -- copy database into the www directory
    if os.exists(path.websrc) == true and os.exists(path.webdst) == true and os.exists('packages.db') == true then
      os.execute("cp -Rp "..path.websrc.."/* "..path.webdst.."/")
      os.execute("cp -f packages.db "..path.webdst.."/")
    else
      print("Warning: can't finalize web action!")
    end
  end

end
