--
-- configuration.lua
-- load configuration for the debian architecture
-- Copyright (c) 2010 pasgui
--
-- Hash: $Format:%h$
-- Date: $Format:%ci$
--

-----------------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------------
module('configuration', package.seeall)

-----------------------------------------------------------------------------
-- Variables declaration
-----------------------------------------------------------------------------

repository = nil
path = nil
packages = nil

-----------------------------------------------------------------------------
-- Public functions
-----------------------------------------------------------------------------

function load(filename)

  -- load configuration
  local p = assert(xmlreader.from_file(filename, 'iso-8859-1', {'dtdvalid', 'nonet'}))

  -- find all paths
  path = {}
  -- parse xml configuration file
  while (p:read()) do
    -- node path found
    if p:name() == 'path' then
      -- parse all attributes
      while (p:move_to_next_attribute()) do
        path[p:name()] = p:value()
      end
      -- stop parsing; only path node
      break
    end
  end

  -- find repository informations
  repository = {}
  -- parse xml configuration file
  while (p:read()) do
    -- node repository found
    if p:name() == 'repository' then
      -- parse all attributes
      while (p:move_to_next_attribute()) do
        repository[p:name()] = p:value()
      end
      -- stop parsing; only repository node
      break
    end
  end

  -- find all packages
  packages = {}
  local package = ''
  local architecture = ''
  -- parse xml configuration file
  while (p:read()) do
    -- node package found
    if p:name() == 'package' and p:node_type() == 'element' then
      package = p:get_attribute('name')
      packages[package] = {}
      -- parse all attributes
      local b = {}
      while (p:move_to_next_attribute()) do
        b[p:name()] = p:value()
      end
      setmetatable(packages[package], { __index = b } )
    -- node architecture
    elseif p:name() == 'architecture' and p:node_type() == 'element' then
      architecture = p:get_attribute('name')
      packages[package][architecture] = {}
      local b = {name = architecture}
      setmetatable(packages[package][architecture], { __index = b } )
    -- node deb
    elseif p:name() == 'deb' and p:node_type() == 'element' then
      -- parse all attributes
      local t = {}
      while (p:move_to_next_attribute()) do
        t[p:name()] = p:value()
      end
      table.insert(packages[package][architecture], t)
    end
  end

  if p:is_valid() == false then
    error("Error: xml configuration '"..filename.."' isn't correct!", 0)
  end

  -- ubuntu directory
  path.ubuntu = _OPTIONS["ubuntu"] or path.ubuntu

  -- pool directory
  path.pool = _OPTIONS["pool"] or path.pool
  if os.exists(path.pool) == false then
    error("Error: pool directory '"..path.pool.."' not found!", 0)
  end

  -- web destination directory
  path.webdst = _OPTIONS["webdst"] or path.webdst or path.ubuntu

  -- web source directory
  path.websrc = _OPTIONS["websrc"] or path.websrc or "www"

end
