--
-- \file   debian.lua
-- \brief  debian package module
-- \author Copyright (c) 2010 pasgui
--
-- Hash: $Format:%h$
-- Date: $Format:%ci$
--

-----------------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------------
module('debian', package.seeall)

-----------------------------------------------------------------------------
-- Variables declaration
-----------------------------------------------------------------------------

local tag = { ["Package"]="name",  ["Binary"]="binary", ["Version"]="version",  ["Priority"]="priority",  ["Section"]="section",
              ["Maintainer"]="maintainer",  ["Original-Maintainer"]="originalmaintainer", ["Build-Depends"]="builddepends"; ["Architecture"]="architecture";
              ["Standards-Version"]="standardsversion"; ["Format"]="format"; ["Directory"]="directory"; ["Files"]="files";
              ["Installed-Size"]="installedsize",  ["Source"]="source", ["Depends"]="depends"; ["Filename"]="filename";
              ["Size"]="size",  ["MD5sum"]="md5sum", ["SHA1"]="sha1"; ["SHA256"]="sha256"; ["Description"]="description";
              ["Homepage"]="homepage"; ["Checksums-Sha1"]="chescksumsha1"; ["Checksums-Sha512"]="chescksumsha512"; ["Checksums-Sha256"]="chescksumsha256"; ["Package-List"]="package-list";
              ["Conflicts"]="conflicts"; ["Breaks"]="breaks"; ["Replaces"]="replaces"; ["Pre-Depends"]="pre-depends";
              ["Build-Depends-Indep"]="build-depends-indep"; ["Uploaders"]="uploaders"; ["Vcs-Browser"]="vcs-browser"; ["Vcs-Svn"]="vcs-svn";
              ["Code"]="code"; ["SHA512"]="sha512"; ["Recommends"]="recommends"; ["Suggests"]="suggests"}

local state = nil

local debianpackage = {}

-----------------------------------------------------------------------------
-- Public functions
-----------------------------------------------------------------------------

function packages(filename)

  -- cheks if the file exists before continue
  assert(os.exists(filename), "file '"..filename.."' doesn't exist.")

  -- coroutine function
  local function yieldpackages(filename)
    for line in bz2.open(filename):lines() do
      line = line:trim()
      local _, _, key, value =line:find("^([^: ]*):(.*)")
      -- debian fields found
      if key ~= nil then
        if tag[key] ~= nil then
          -- new package description
          if key == "Package" then
            debianpackage = {}
            debianpackage.longdescription = ""
          end
          -- save current value associated with this field except "files" field
          if key ~= "Files" then
            debianpackage[tag[key]] = value:trim()
          else
            debianpackage.files = {}
          end
          -- multi-lines fields
          if string.find("Description Files", key, 1, true) ~= nil then
            state = tag[key]
          else
            state = nil
          end
        else
          print('unknown debian key:', key)
        end
      -- empty line
      elseif line == "" then
        -- end of package
        if debianpackage.name ~= nil then
          if debianpackage.source == nil then
            debianpackage.source = debianpackage.name
          end
          coroutine.yield(debianpackage)
          debianpackage = {}
        end
      -- others: multi-lines
      else
        -- Files
        if state == tag["Files"] then
          local _, _, hash, size, filename =line:find("%s*([^%s]+)%s+([^%s]+)%s+([^%s]+)")
          table.insert(debianpackage.files, {hash, size, filename})
        end
        -- Description
        if state == tag["Description"] and line ~= "." then
          debianpackage.longdescription = debianpackage.longdescription .. line:gsub('([\'%"])', "%1%1") .. "\n"
        end
      end
    end
  end

  return coroutine.wrap(function() yieldpackages(filename) end)

end


