--
-- repository.lua
-- repository generator module
-- Copyright (c) 2010 pasgui
--
-- Hash: $Format:%h$
-- Date: $Format:%ci$
--

-----------------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------------
module('repository', package.seeall)

-----------------------------------------------------------------------------
-- Variables declaration
-----------------------------------------------------------------------------

local filelist, override, repository, path, packages

-----------------------------------------------------------------------------
-- Private functions
-----------------------------------------------------------------------------

--
-- Create architecture directory
--
local function create_architecture()
  -- commands
  local preparerepository = [[
    mkdir -p dists
    mkdir -p cache
    rm -rf dists/*
    rm -rf _REPOSITORY_PATH_
    mkdir -p _REPOSITORY_PATH_/dists
    mkdir -p _REPOSITORY_PATH_/indices
  ]]
  os.execute(string.gsub(preparerepository, "_REPOSITORY_PATH_", path.ubuntu))
  filelist = {}
  for distribution, _ in repository.distributions:gmatch("%w+") do
    lfs.mkdir(path.ubuntu.."/dists/" .. distribution)
    filelist[distribution] = {}
    for component, _ in repository.components:gmatch("%w+") do
      lfs.mkdir(path.ubuntu.."/dists/" .. distribution .. "/" .. component)
      filelist[distribution][component] = {}
      -- create architecture directory
      for architecture, _ in repository.architectures:gmatch("%w+") do
        lfs.mkdir(path.ubuntu.."/dists/" .. distribution .. "/" .. component .. "/binary-" .. architecture)
        filelist[distribution][component][architecture] = {}
      end
      -- create source directory
      lfs.mkdir(path.ubuntu.."/dists/" .. distribution .. "/" .. component .. "/source")
      filelist[distribution][component]["source"] = {}
    end
  end
  override = {}
  for component, _ in repository.components:gmatch("%w+") do
    override[component] = {}
    override[component]['source'] = {}
    override[component]['package'] = {}
  end
end

--
-- check the modifcation date
--
local function check_ext(ext, file)
  local insert = true
  -- condition on the modification date
  if ext ~= nil and ext ~= '' then
    local attr = lfs.attributes (file)
    -- date doesn't math the condition or error in the condition format
    insert = date.compare(attr["modification"], ext)
  end
  return insert
end

--
-- get relative pool directory with
-- ubuntu directory
--
local function get_relative_pool()
  local currentdir = lfs.currentdir()
  lfs.chdir(path.ubuntu)
  local fullubuntupath = lfs.currentdir()
  lfs.chdir(currentdir)
  local poolpath = path.pool:gsub("(.*)/.*", "%1")
  local poolname = path.pool:gsub(".*/(.*)", "%1")
  lfs.chdir(poolpath)
  local fullpoolpath = lfs.currentdir()
  lfs.chdir(currentdir)
  local path = ''
  while fullpoolpath:find(fullubuntupath, 1, true) == nil do
    fullubuntupath = fullubuntupath:gsub("(.*)/.*", "%1")
    path = path .. '../'
  end
  _, n = fullpoolpath:find(fullubuntupath, 1, true)
  return path..fullpoolpath:sub(n+2)..poolname..'/'
end

--
-- generate packages file list
--
local function create_packages_list()
  -- parse all packages
  for _, package in pairs(packages) do
    local pool = get_relative_pool()..package.pool
    local component = package.component
    -- build override
    override[component]['source'][package] = {}
    table.insert(override[component]['source'][package], package.name .. '\t' .. package.component .. '/' .. package.section)
    -- parse all architectures for the current package
    for _, architecture in pairs(package) do
      -- parse all deb files for the current architecture
      for _, deb in pairs(architecture) do
        -- list all files into the pool
        local pattern_is_found = false
        for file in lfs.dir(path.ubuntu..'/'..pool) do
          -- file match the pattern
          if file:find("^"..deb.name.."$") ~= nil then
            pattern_is_found = true;
            -- can insert the file
            if check_ext(deb.ext, path.ubuntu..'/'..pool..'/'..file) == true then
              local debfile = file:gsub("^([^_]*).*$", "%1")
              -- build override
              if override[component]['package'][debfile] == nil then
                override[component]['package'][debfile] = {}
                table.insert(override[component]['package'][debfile], debfile .. '\t' .. package.priority .. '\t' .. package.component .. '/' .. package.section .. '\t' .. repository.maintainer)
              end
              -- into the distribution(s)
              for distribution, _ in deb.distribution:gmatch("%w+") do
                table.insert(filelist[distribution][component][architecture.name], pool.."/"..file)
              end -- distribution
            end -- cant insert the file
          end -- file match pattern
        end -- list all files
        if pattern_is_found == false then
          print("Warning: No package with the pattern '"..deb.name.."' found in the pool!")
        end
      end -- debs
    end  -- architectures
  end -- packages
end

--
-- save filelist configuration table
--
local function save_packages_list()
  -- generate package list
  for distribution, _ in repository.distributions:gmatch("%w+") do
    for component, _ in repository.components:gmatch("%w+") do
      -- create architecture directory
      for architecture, _ in repository.architectures:gmatch("%w+") do
        local file = assert(io.open("dists/filelist-"..distribution.."-"..component.."-"..architecture..".cfg", "w"))
        for _ , debname in pairs(filelist[distribution][component][architecture]) do
          file:write(debname.."\n")
        end
        file:close()
      end
      -- create source directory
      local file = assert(io.open("dists/filelist-"..distribution.."-"..component.."-source.cfg", "w"))
      for _ , debname in pairs(filelist[distribution][component]["source"]) do
        file:write(debname.."\n")
      end
      file:close()
    end
  end
  -- generate override file
  for component, _ in repository.components:gmatch("%w+") do
    local file = assert(io.open("dists/override-"..component.."-source.cfg", "w"))
    for _ , debname in pairs(override[component]['source']) do
      file:write(debname[1].."\n")
    end
    file:close()
    local file = assert(io.open("dists/override-"..component..".cfg", "w"))
    for _ , debname in pairs(override[component]['package']) do
      file:write(debname[1].."\n")
    end
    file:close()
  end
end

--
-- copy repository needed files
--
local function copy_files()
  local release = "Archive: %s\nComponent: %s\nOrigin: %s\nLabel: %s\nArchitectures: %s"
  local override = "cp dists/override-%s.cfg %s/indices/override.%s.%s"
  for distribution, _ in repository.distributions:gmatch("%w+") do
    for component, _ in repository.components:gmatch("%w+") do
      -- all architectures
      for architecture, _ in repository.architectures:gmatch("%w+") do
        local file = assert(io.open(path.ubuntu.."/dists/"..distribution.."/"..component.."/binary-"..architecture.."/Release", "w"))
        file:write(string.format(release, distribution, component, repository.origin, repository.label, architecture))
        file:close()
      end
      -- source
      local file = assert(io.open(path.ubuntu.."/dists/"..distribution.."/"..component.."/source/Release", "w"))
        file:write(string.format(release, distribution, component, repository.origin, repository.label, "source"))
      file:close()
      -- override
      os.execute(string.format(override, component, path.ubuntu, distribution, component))
      os.execute(string.format(override, component.."-source", path.ubuntu, distribution, component.."-source"))
    end
  end
end

--
-- complete apt-ftparchive.conf file with distributions
--
local function complete_apt_file()
  local ftparchive = [[
  Dir {
    ArchiveDir "_REPOSITORY_PATH_";
    CacheDir "cache";
    OverrideDir "dists";
    FileListDir "dists";
    SourceFileListDir "dists";
  };

  Default {
    Packages::Compress ". gzip bzip2";
    Sources::Compress ". gzip bzip2";
    Contents::Compress ". gzip";
    DeLinkLimit 0;
    MaxContentsChange 25000;
    FileMode 0664;
  };

  TreeDefault {
    BinCacheDB "packages-$(DIST)-$(SECTION)-$(ARCH).db";
    Packages "dists/$(DIST)/$(SECTION)/binary-$(ARCH)/Packages";
    Sources "dists/$(DIST)/$(SECTION)/source/Sources";
    Contents "dists/$(DIST)/Contents-$(ARCH)";
    FileList "filelist-$(DIST)-$(SECTION)-$(ARCH).cfg";
    SourceFileList "filelist-$(DIST)-$(SECTION)-source.cfg";
  };

  ]]
  ftparchive = string.gsub(ftparchive, "_REPOSITORY_PATH_", path.ubuntu)
  local ftparchivedist = [[
  Tree "%s" {
    Sections "%s";
    Architectures "%s source";
    BinOverride "override-$(SECTION).cfg";
    SrcOverride "override-$(SECTION)-source.cfg";
  }

  ]]
  local f = assert(io.open("dists/apt-ftparchive.conf", "w"))
  f:write(ftparchive)
  for distribution, _ in repository.distributions:gmatch("%w+") do
    f:write(string.format(ftparchivedist, distribution, repository.components, repository.architectures))
  end
  f:close()
end

--
-- clean and build repository
-- generate index file
--
local function clean_and_build()
  -- clean and build repository
  os.execute[[
  apt-ftparchive clean dists/apt-ftparchive.conf
  apt-ftparchive generate dists/apt-ftparchive.conf
  ]]
  -- generate index file
  print("Generate index file ...")
  local aptcustom = [[
  APT {
    FTPArchive {
      Release {
        Origin "%s";
        Label "%s";
        Suite "%s";
        Codename "%s";
        Components "%s";
        Architectures "%s source";
      };
    };
  };
  ]]
  local release = [[
  apt-ftparchive -c dists/apt-custom-release.conf release _DIST_PATH_ > _DIST_PATH_/Release
  gpg --output _DIST_PATH_/Release.gpg -ba _DIST_PATH_/Release
  ]]
  for distribution, _ in repository.distributions:gmatch("%w+") do
    local f = assert(io.open("dists/apt-custom-release.conf", "w"))
    f:write(string.format(aptcustom, repository.origin, repository.label, distribution.."-"..repository.suffix, distribution, repository.components, repository.architectures))
    f:close()
    os.execute(string.gsub(release, "_DIST_PATH_", path.ubuntu.."/dists/"..distribution))
  end
end

--
-- finalize repository
--
local function finalize()
  -- remove plain text files
  print("Remove plain text files ...")
  for architecture, _ in repository.architectures:gmatch("%w+") do
    os.execute("find "..path.ubuntu.." -type f -name Contents-"..architecture.." -exec rm -f {} \\;")
  end
  os.execute("find "..path.ubuntu.." -type f -name Packages -exec rm -f {} \\;")
  os.execute("find "..path.ubuntu.." -type f -name Sources -exec rm -f {} \\;")

  -- create ls-lR file
  print("Create ls-lR file ...")
  os.execute("cd "..path.ubuntu.." && ls -lR | gzip -9 > ls-lR.gz")

  -- create md5sum file
  print("Create md5sum file ...")
  os.execute("find "..path.ubuntu.." -type f 2>/dev/null -exec md5sum {} \\; | gzip -9 > "..path.ubuntu.."/indices/md5sums.gz")
end

-----------------------------------------------------------------------------
-- Public functions
-----------------------------------------------------------------------------

--
-- build repository
--
function build(filename)

  -- get path, packages and repository information from configuration
  repository = configuration.repository
  path = configuration.path
  packages = configuration.packages

  print("Create architecture directory ...")
  create_architecture()

  print("Generate source/package file list ...")
  create_packages_list()

  print("Save file list configuration ...")
  save_packages_list()

  print("Copy repository needed files ...")
  copy_files()

  print("Complete apt-ftparchive.conf file with distributions ...")
  complete_apt_file()

  print("clean and build architecture")
  clean_and_build()

  print("cleaning ...")
  finalize()
end
