--
-- web.lua
-- web generator module
-- Copyright (c) 2010 pasgui
--
-- Hash: $Format:%h$
-- Date: $Format:%ci$
--

-----------------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------------
module('web', package.seeall)

-----------------------------------------------------------------------------
-- Variables declaration
-----------------------------------------------------------------------------

local repository, path

-----------------------------------------------------------------------------
-- Private functions
-----------------------------------------------------------------------------

--
-- parse all sources and packages files
--
local function parse()
  for distribution, _ in repository.distributions:gmatch("%w+") do
    print(distribution)
    for component, _ in repository.components:gmatch("%w+") do
      print("  " .. component)
      -- parse sources files
      local sourcefilename = path.ubuntu .. "/dists/" .. distribution .. "/" .. component .. "/source/Sources.bz2";
      if os.exists(sourcefilename) then
        print("    source: " .. sourcefilename)
        -- commit sources file parsed
        dbm:exec("BEGIN TRANSACTION;")
        for debianpackage in debian.packages(sourcefilename) do
          for _, attr in pairs(debianpackage.files) do
            md5sum, size, file = table.unpack(attr)
            local _, _, version, extension = debianpackage.version:find("(.*)-(.*)")
            assert(dbm:exec("INSERT OR REPLACE INTO files VALUES('"..debianpackage.source.."', '"..version.."', 'source', '"..debianpackage.directory.."/', '"..extension.."', '"..debianpackage.name.."', '"..distribution.."', '"..file.."', "..size..", '"..component.."', '"..debianpackage.section.."', 1);"))
          end
        end
        dbm:exec("COMMIT;")
      else
        print(sourcefilename .. " doesn't exist")
      end
      -- parse packages files
      for architecture, _ in repository.architectures:gmatch("%w+") do
        local sourcefilename = path.ubuntu .. "/dists/" .. distribution .. "/" .. component .. "/binary-" .. architecture .. "/Packages.bz2";
        if os.exists(sourcefilename) then
          print("    " .. architecture .. ": " .. sourcefilename)
          -- commit packages file parsed
          dbm:exec("BEGIN TRANSACTION;")
          for debianpackage in debian.packages(sourcefilename) do
            local longdescription = debianpackage.longdescription:gsub("\n[-*] ", "<br/>- "):gsub("\n", " ")
            local description = debianpackage.description:gsub("\n", "<br/>")
            local _, _, directory, file =debianpackage.filename:find("(.*)/(.*%.deb)")
            local _, _, version, extension = debianpackage.version:find("(.*)-(.*)")
            local common_architecture
            if file:match('.*_all.deb') == nil then
              all_architecture = 0
            else
              all_architecture = 1
            end
            assert(dbm:exec("INSERT OR REPLACE INTO sources VALUES('"..debianpackage.name.."', '"..description.."', '"..longdescription.."', '"..debianpackage.source.."');"))
            assert(dbm:exec("INSERT OR REPLACE INTO files VALUES('"..debianpackage.source.."', '"..version.."', '"..architecture.."', '"..directory.."/', '"..extension.."', '"..debianpackage.name.."', '"..distribution.."', '"..file.."', "..debianpackage.size..", '"..component.."', '"..debianpackage.section.."', "..all_architecture..");"))
          end
          dbm:exec("COMMIT;")
        else
          print(sourcefilename .. " doesn't exist")
        end
      end
    end
  end
end

--
-- generate sources table
--
local function generate_source_file()
  db:exec("BEGIN TRANSACTION;")
  local stmt = assert(dbm:prepare("SELECT * FROM sources WHERE source=:name"))
  for row_source in dbm:nrows("SELECT DISTINCT source FROM 'sources' ORDER BY source ASC") do
    stmt:bind_names({name=row_source.source})
    if stmt:step() == sqlite3.ROW then
      row = stmt:get_named_values()
      assert(db:exec('INSERT INTO sources VALUES("' .. row_source.source .. '", "' .. row.description .. '", "' .. row.longdescription .. '");'))
    else
      print("\n\tError while generating sources table\n");
    end
    stmt:reset()
  end
  db:exec("COMMIT;")
end

--
-- generate packages table
--
local function generate_packages_table()
  db:exec("BEGIN TRANSACTION;")
  for row in dbm:nrows("SELECT DISTINCT * FROM 'sources'") do
    assert(db:exec('INSERT INTO packages VALUES("' .. row.name .. '", "' .. row.description .. '", "' .. row.source .. '");'))
  end
  db:exec("COMMIT;")
end
--
-- generate version table
--
local function generate_version_table()
  db:exec("BEGIN TRANSACTION;")
  local stmt = assert(dbm:prepare("SELECT * FROM 'files' WHERE package=? AND version=? AND section=? AND filename GLOB ?"))
  for row in dbm:nrows("SELECT DISTINCT package, version, section FROM 'files' ORDER BY package ASC") do
    -- find all deb file included for the current packages
    stmt:bind_values(row.package, row.version, row.section, "*.deb")
    if stmt:step() == sqlite3.ROW then
      row_file = stmt:get_named_values()
      -- package directory exists
      if os.exists(path.ubuntu .. "/" .. row_file.path) then
        -- find current version
        local version = string.gsub(row.version, "%w*:(%w+)", "%1")
        -- find at least one deb file in the pool for the current package
        local filename = os.capture("cd " .. path.ubuntu .. " && ls -t " .. row_file.path .. " | grep -m1 --regexp='" .. row_file.package .. "_" .. version .. "[-|_].*.deb'")
        if filename ~= '' then
          -- search most recent modification date for the current package
          published = assert(os.capture("cd " .. path.ubuntu .. " && date --rfc-3339=date -r " .. row_file.path .. filename))
          -- search most recent changes file for the current package
          local changes = os.capture("cd " .. path.ubuntu .. "/" .. row_file.path .. " && ls "..row_file.package .."_" .. version .. "*.changes | tail -1")
          local timestamp = 0
          if changes ~= '' then
            timestamp = os.capture("cd " .. path.ubuntu .. "/" .. row_file.path .. " && stat -c %Y "..changes)
            changes = row_file.path .. changes
          end
          -- find distribution for the current package
          local distributions = ''
          for row_dist in dbm:nrows("SELECT DISTINCT distribution FROM 'files' WHERE package='" .. row.package .. "' AND version='" .. row.version .. "' AND section='" .. row.section .. "'") do
            distributions = distributions .. " " .. row_dist.distribution
          end
          -- find architecture for the current package
          local architectures = ''
          for row_arch in dbm:nrows("SELECT DISTINCT architecture FROM 'files' WHERE package='" .. row.package .. "' AND version='" .. row.version .. "' AND section='" .. row.section .. "' AND all_architecture=0") do
              architectures = architectures .. " " .. row_arch.architecture
          end
          if architectures == "" then
            architectures = "all"
          end
          -- insert into the database
          db:exec("INSERT INTO 'version' VALUES('" .. row.package .. "', '" .. row.version .. "', '" .. distributions .. "', '" .. architectures .. "', '" .. published .."', '" .. row.section .."', '" .. changes .."'," .. timestamp .. ");")
        else
          print(row.package, row.version, row.section .. ": No *.deb files ("..dbm:error_message()..")");
        end
      else
        print(row.package, row.version, row.section .. ": Directory '"..path.ubuntu .. "/" .. row_file.path.."' doesn't exist");
      end
    else
       print(row.package, row.version, row.section .. ": No *.deb files ("..dbm:error_message()..")");
    end
    stmt:reset()
  end
  db:exec("COMMIT;")
end

--
-- generate files table
--
local function generate_files_table()
  db:exec("BEGIN TRANSACTION;")
  local stmt = assert(dbm:prepare("SELECT * FROM 'files' WHERE filename=? AND size=? AND architecture=?;"))
  local insert_stmt  = assert(db:prepare("INSERT INTO 'files' VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);"))
  for row_filename in dbm:nrows("SELECT DISTINCT package, filename, size, architecture FROM 'files' ORDER BY package ASC, filename ASC") do
    -- find all distribution linked with the same file
    stmt:reset()
    stmt:bind_values(row_filename.filename, row_filename.size, row_filename.architecture)
    local dist = ''
    for row in stmt:nrows() do
      dist = dist .. " " .. row.distribution
    end
    -- get first item
    stmt:reset()
    stmt:step()
    row = stmt:get_named_values()
    -- compute size
    if row.size > 1048576 then
      size = string.format('%.2f MiB', row.size/1048576)
    elseif row.size > 1024 then
      size = string.format('%.2f KiB', row.size/1024)
    else
      size = string.format('%d B', row.size)
    end
    -- insert filename into the database
    insert_stmt:reset()
    insert_stmt:bind_values(row.package, row.version, row.architecture, row.path, row.extension, row.deb, dist, row.filename, size, row.repository, row.section)
    insert_stmt:step()
  end
  db:exec("COMMIT;")
end

-----------------------------------------------------------------------------
-- Public functions
-----------------------------------------------------------------------------

--
-- build web folder
--
function build()

  -- get path and repository information from configuration
  repository = configuration.repository
  path = configuration.path

  -- remove previous database
  os.remove('packages.db')

  -- open database
  db = sqlite3.open('packages.db')

  -- create all tables into the database
  db:exec[[
    BEGIN TRANSACTION;
    CREATE TABLE 'sources' ('source' TEXT, 'description' TEXT, 'longdescription' TEXT);
    CREATE TABLE 'packages' ('name' TEXT unique, 'description' TEXT, 'source' TEXT);
    CREATE TABLE 'files' ('package' TEXT, 'version' TEXT, 'architecture' TEXT, 'path' TEXT, 'extension' TEXT, 'deb' TEXT, 'distribution' TEXT, 'filename' TEXT, 'size' INTEGER, 'repository' TEXT, 'section' TEXT);
    CREATE TABLE 'version' ('name' TEXT, 'version' TEXT, 'distribution' TEXT, 'architecture' TEXT, 'date' TEXT, 'section' TEXT, 'changes' TEXT, 'timestamp' INTEGER);
    COMMIT;
  ]]

  -- open memory database
  dbm = sqlite3.open_memory()

  -- create all tables into the database
  dbm:exec[[
    BEGIN TRANSACTION;
    CREATE TABLE 'sources' ('name' TEXT unique, 'description' TEXT, 'longdescription' TEXT, 'source' TEXT);
    CREATE TABLE 'files' ('package' TEXT, 'version' TEXT, 'architecture' TEXT, 'path' TEXT, 'extension' TEXT, 'deb' TEXT, 'distribution' TEXT, 'filename' TEXT, 'size' INTEGER, 'repository' TEXT, 'section' TEXT, 'all_architecture' INTEGER);
    COMMIT;
  ]]

  print("Parse all sources and packages files ...")
  parse()

  print("Generate sources table ...")
  generate_source_file()

  print("Generate packages table ...")
  generate_packages_table()

  print("Generate version table ...")
  generate_version_table()

  print("Generate files table ...")
  generate_files_table()

  -- close the databases
  assert(db:close() == sqlite3.OK)
  assert(dbm:close() == sqlite3.OK)
end
